@@ -1,4 +1,4 @@
-/* $Id: login.c 3344 2007-07-10 14:25:55Z fpeters $
+/* $Id: login.c 3725 2008-05-21 17:28:44Z dlaniel $
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -22,6 +22,127 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/**
+ * SECTION:login
+ * @short_description: Single Sign-On and Federation Profile
+ *
+ * The Single Sign On process allows a user to log in once to an identity
+ * provider (IdP), and to be then transparently loged in to the required
+ * service providers (SP) belonging to the IP "circle of trust".  Subordinating
+ * different identities of the same user within a circle of trust to a unique
+ * IP is called "Identity Federation".  The liberty Alliance specifications
+ * allows, thanks to this federation, strong and unique authentication coupled
+ * with control by the user of his personal informations. The explicit user
+ * agreement is necessary before proceeding to Identity Federation.
+ *
+ * <para>
+ * The service provider must implement the following process:
+ * <itemizedlist>
+ *  <listitem><para>creating an authentication request (#LassoLibAuthnRequest) with
+ *  lasso_login_init_authn_request();</para></listitem>
+ *  <listitem><para>sending it to the identity provider with
+ *  lasso_login_build_authn_request_msg();</para></listitem>
+ *  <listitem><para>receiving and processing the answer:
+ *    <itemizedlist>
+ *      <listitem>either an authentication response with
+ *      lasso_login_process_authn_response_msg()</listitem>
+ *      <listitem>or an artifact with lasso_login_init_request() then sending the
+ *      request to the IdP with lasso_login_build_request_msg() and processing the
+ *      new answer with lasso_login_process_response_msg().</listitem>
+ *    </itemizedlist>
+ *    </para></listitem>
+ * </itemizedlist>
+ * </para> 
+ * <example>
+ * <title>Service Provider Login URL</title>
+ * <programlisting>
+ * LassoLogin *login;
+ * 
+ * login = lasso_login_new(server);
+ * lasso_login_init_authn_request(login, "http://identity-provider-id/",
+ *                 LASSO_HTTP_METHOD_REDIRECT);
+ * 
+ * // customize AuthnRequest
+ * request = LASSO_LIB_AUTHN_REQUEST(LASSO_PROFILE(login)->request);
+ * request->NameIDPolicy = strdup(LASSO_LIB_NAMEID_POLICY_TYPE_FEDERATED);
+ * request->ForceAuthn = TRUE;
+ * request->IsPassive = FALSE;
+ * request->ProtocolProfile = strdup(LASSO_LIB_PROTOCOL_PROFILE_BRWS_ART);
+ * 
+ * lasso_login_build_authn_request_msg(login);
+ * 
+ * // redirect user to identity provider
+ * printf("Location: %s\n\nRedirected to IdP\n", LASSO_PROFILE(login)->msg_url);
+ * </programlisting>
+ * </example>
+ * 
+ * <example>
+ * <title>Service Provider Assertion Consumer Service URL</title>
+ * <programlisting>
+ * LassoLogin *login;
+ * char *request_method = getenv("REQUEST_METHOD");
+ * char *artifact_msg = NULL, *lares = NULL, *lareq = NULL;
+ * char *name_identifier;
+ * lassoHttpMethod method;
+ * 
+ * login = lasso_login_new(server);
+ * if (strcmp(request_method, "GET") == 0) {
+ *         artifact_msg = getenv("QUERY_STRING");
+ *         method = LASSO_HTTP_METHOD_REDIRECT;
+ * } else {
+ *         // read submitted form; if it has a LAREQ field, put it in lareq,
+ *         // if it has a LARES field, put it in lares
+ *         if (lareq) {
+ *                 artifact_msg = lareq;
+ *         } else if (lares) {
+ *                 response_msg = lares;
+ *         } else {
+ *                 // bail out
+ *         }
+ *         method = LASSO_HTTP_METHOD_POST;
+ * }
+ * 
+ * if (artifact_msg) {
+ *         lasso_login_init_request(login, artifact_msg, method);
+ *         lasso_login_build_request_msg(login);
+ *         // makes a SOAP call, soap_call is NOT a Lasso function
+ *         soap_answer_msg = soap_call(LASSO_PROFILE(login)->msg_url,
+ *                         LASSO_PROFILE(login)->msg_body);
+ *         lasso_login_process_response_msg(login, soap_answer_msg);
+ * } else if (response_msg) {
+ *         lasso_login_process_authn_response_msg(login, response_msg);
+ * }
+ * 
+ * // looks up name_identifier in local file, database, whatever and gets back
+ * // two things: identity_dump and session_dump
+ * name_identifier = LASSO_PROFILE(login)->nameIdentifier
+ * lasso_profile_set_identity_from_dump(LASSO_PROFILE(login), identity_dump);
+ * lasso_profile_set_session_from_dump(LASSO_PROFILE(login), session_dump);
+ * 
+ * lasso_login_accept_sso(login);
+ * 
+ * if (lasso_profile_is_identity_dirty(LASSO_PROFILE(login))) {
+ *         LassoIdentity *identity;
+ *         char *identity_dump;
+ *         identity = lasso_profile_get_identity(LASSO_PROFILE(login));
+ *         identity_dump = lasso_identity_dump(identity);
+ *         // record identity_dump in file, database...
+ * }
+ * 
+ * if (lasso_profile_is_session_dirty(LASSO_PROFILE(login))) {
+ *         LassoSession *session;
+ *         char *session_dump;
+ *         session = lasso_profile_get_session(LASSO_PROFILE(login));
+ *         session_dump = lasso_session_dump(session);
+ *         // record session_dump in file, database...
+ * }
+ * 
+ * // redirect user anywhere
+ * printf("Location: %s\n\nRedirected to site root\n", login->msg_url);
+ * </programlisting>
+ * </example>
+ */ 
+
 #include <xmlsec/base64.h>
 
 #include <lasso/lasso_config.h>
@@ -33,6 +154,7 @@
 #include <lasso/xml/saml_audience_restriction_condition.h>
 #include <lasso/xml/saml_conditions.h>
 #include <lasso/xml/samlp_response.h>
+#include <lasso/xml/saml-2.0/saml2_encrypted_element.h>
 
 #ifdef LASSO_WSF_ENABLED
 #include <lasso/xml/disco_description.h>
@@ -203,6 +325,9 @@ lasso_login_build_assertion(LassoLogin *login,
 	LassoSamlNameIdentifier *nameIdentifier = NULL;
 	LassoProfile *profile;
 	LassoFederation *federation;
+	LassoProvider *provider = NULL;
+	LassoSaml2EncryptedElement *encrypted_element = NULL;
+	LassoSamlSubjectStatementAbstract *ss;
 
 	g_return_val_if_fail(LASSO_IS_LOGIN(login), LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
 
@@ -225,7 +350,8 @@ lasso_login_build_assertion(LassoLogin *login,
 			LASSO_SAMLP_REQUEST_ABSTRACT(profile->request)->RequestID,
 			profile->remote_providerID, notBefore, notOnOrAfter));
 
-	if (strcmp(login->nameIDPolicy, LASSO_LIB_NAMEID_POLICY_TYPE_ONE_TIME) == 0) {
+	if (strcmp(login->nameIDPolicy, LASSO_LIB_NAMEID_POLICY_TYPE_ONE_TIME) == 0 ||
+			federation == NULL) {
 		/* if NameIDPolicy is 'onetime', don't use a federation */
 		nameIdentifier = lasso_saml_name_identifier_new();
 		nameIdentifier->content = lasso_build_unique_id(32);
@@ -244,6 +370,21 @@ lasso_login_build_assertion(LassoLogin *login,
 				LASSO_SAML_NAME_IDENTIFIER(federation->local_nameIdentifier));
 	}
 
+	/* Encrypt NameID */
+	provider = g_hash_table_lookup(profile->server->providers, profile->remote_providerID);
+	ss = LASSO_SAML_SUBJECT_STATEMENT_ABSTRACT(as);
+	if (provider && provider->private_data->encryption_mode & LASSO_ENCRYPTION_MODE_NAMEID
+			&& provider->private_data->encryption_public_key != NULL) {
+		encrypted_element = LASSO_SAML2_ENCRYPTED_ELEMENT(lasso_node_encrypt(
+			LASSO_NODE(ss->Subject->NameIdentifier),
+			provider->private_data->encryption_public_key,
+			provider->private_data->encryption_sym_key_type));
+		if (encrypted_element != NULL) {
+			ss->Subject->EncryptedNameIdentifier = encrypted_element;
+			ss->Subject->NameIdentifier = NULL;
+		}
+	}
+
 	assertion->AuthenticationStatement = LASSO_SAML_AUTHENTICATION_STATEMENT(as);
 
 	if (profile->server->certificate) {
@@ -258,7 +399,8 @@ lasso_login_build_assertion(LassoLogin *login,
 	if (login->protocolProfile == LASSO_LOGIN_PROTOCOL_PROFILE_BRWS_POST || \
 			login->protocolProfile == LASSO_LOGIN_PROTOCOL_PROFILE_BRWS_LECP) {
 		/* only add assertion if response is an AuthnResponse */
-		LASSO_SAMLP_RESPONSE(profile->response)->Assertion = g_list_append(NULL, assertion);
+		LASSO_SAMLP_RESPONSE(profile->response)->Assertion = g_list_append(NULL,
+				g_object_ref(assertion));
 	}
 
 	lasso_login_assertion_add_discovery(login, assertion);
@@ -269,14 +411,13 @@ lasso_login_build_assertion(LassoLogin *login,
 	}
 	if (login->assertion)
 		lasso_node_destroy(LASSO_NODE(login->assertion));
-	login->assertion = LASSO_SAML_ASSERTION(g_object_ref(assertion));
+	login->assertion = LASSO_SAML_ASSERTION(assertion);
 	lasso_session_add_assertion(profile->session, profile->remote_providerID,
-			g_object_ref(assertion));
+			LASSO_NODE(assertion));
 
 	if (LASSO_SAMLP_REQUEST_ABSTRACT(profile->request)->MajorVersion == 1 && 
 			LASSO_SAMLP_REQUEST_ABSTRACT(profile->request)->MinorVersion < 2) {
 		/* pre-id-ff 1.2, saml 1.0 */
-		LassoSamlSubjectStatementAbstract *ss;
 
 		/* needs assertion artifact */
 		lasso_login_build_assertion_artifact(login);
@@ -479,6 +620,11 @@ lasso_login_process_response_status_and_assertion(LassoLogin *login)
 	LassoProvider *idp;
 	LassoSamlpResponse *response;
 	char *status_value;
+	LassoSamlSubjectStatementAbstract *sssa = NULL;
+	LassoSamlSubjectStatementAbstract *sas = NULL;
+	LassoNode *encrypted_id = NULL;
+	LassoSaml2EncryptedElement* encrypted_element = NULL;
+	xmlSecKey *encryption_private_key = NULL;
 	int ret = 0;
 
 	g_return_val_if_fail(LASSO_IS_LOGIN(login), LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
@@ -525,24 +671,57 @@ lasso_login_process_response_status_and_assertion(LassoLogin *login)
 
 		/* store NameIdentifier */
 		if (assertion->AuthenticationStatement) {
-			LassoSamlSubjectStatementAbstract *sssa;
 			sssa = LASSO_SAML_SUBJECT_STATEMENT_ABSTRACT(
 					assertion->AuthenticationStatement);
 			if (sssa->Subject && sssa->Subject->NameIdentifier) {
 				profile->nameIdentifier = g_object_ref(
 						sssa->Subject->NameIdentifier);
+			} else {
+				encrypted_id = g_object_ref(sssa->Subject->EncryptedNameIdentifier);
 			}
 		}
 
-		if (profile->nameIdentifier == NULL) {
+		if (profile->nameIdentifier == NULL && encrypted_id == NULL) {
 			/* it was not found in AuthenticationStatement, look it
 			 * up in saml:AttributeStatement */
-			LassoSamlSubjectStatementAbstract *sas;
-
 			sas = LASSO_SAML_SUBJECT_STATEMENT_ABSTRACT(assertion->AttributeStatement);
 			if (sas->Subject && sas->Subject->NameIdentifier) {
 				profile->nameIdentifier = g_object_ref(
 						sas->Subject->NameIdentifier);
+			} else {
+				encrypted_id = g_object_ref(sas->Subject->EncryptedNameIdentifier);
+			}
+		}
+
+		if (profile->nameIdentifier == NULL && encrypted_id == NULL) {
+			return LASSO_PROFILE_ERROR_NAME_IDENTIFIER_NOT_FOUND;
+		}
+
+		if (profile->nameIdentifier != NULL) {
+			return ret;
+		}
+
+		encrypted_element = LASSO_SAML2_ENCRYPTED_ELEMENT(encrypted_id);
+		encryption_private_key =
+			LASSO_PROFILE(profile)->server->private_data->encryption_private_key;
+		if (encrypted_element != NULL && encryption_private_key == NULL) {
+			return LASSO_PROFILE_ERROR_MISSING_ENCRYPTION_PRIVATE_KEY;
+		}
+
+		/* Decrypt NameID */
+		if (encrypted_element != NULL && encryption_private_key != NULL) {
+			profile->nameIdentifier = LASSO_NODE(
+				lasso_node_decrypt(encrypted_element, encryption_private_key));
+			if (sssa != NULL && sssa->Subject != NULL) {
+				sssa->Subject->NameIdentifier = LASSO_SAML_NAME_IDENTIFIER(
+					profile->nameIdentifier);
+				g_object_unref(sssa->Subject->EncryptedNameIdentifier);
+				sssa->Subject->EncryptedNameIdentifier = NULL;
+			} else if (sas != NULL && sas->Subject != NULL) {
+				sas->Subject->NameIdentifier = LASSO_SAML_NAME_IDENTIFIER(
+					profile->nameIdentifier);
+				g_object_unref(sas->Subject->EncryptedNameIdentifier);
+				sas->Subject->EncryptedNameIdentifier = NULL;
 			}
 		}
 
@@ -603,7 +782,7 @@ lasso_login_accept_sso(LassoLogin *login)
 		return LASSO_PROFILE_ERROR_MISSING_ASSERTION;
 
 	lasso_session_add_assertion(profile->session, profile->remote_providerID,
-			g_object_ref(assertion));
+			LASSO_NODE(assertion));
 
 	authentication_statement = LASSO_SAML_SUBJECT_STATEMENT_ABSTRACT(
 			assertion->AuthenticationStatement);
@@ -1502,6 +1681,11 @@ lasso_login_must_authenticate(LassoLogin *login)
 		 * assertions, a session is enough
 		 */
 		matched = (profile->session != NULL);
+		if (matched) {
+			matched = profile->remote_providerID 
+				&& lasso_session_get_assertion(profile->session, 
+					profile->remote_providerID) != NULL;
+		}
 	}
 	g_list_free(assertions);
 
@@ -1860,8 +2044,10 @@ get_xmlNode(LassoNode *node, gboolean lasso_dump)
 
 	if (login->protocolProfile == LASSO_LOGIN_PROTOCOL_PROFILE_BRWS_ART)
 		xmlNewTextChild(xmlnode, NULL, (xmlChar*)"ProtocolProfile", (xmlChar*)"Artifact");
-	if (login->protocolProfile == LASSO_LOGIN_PROTOCOL_PROFILE_BRWS_POST)
+	else if (login->protocolProfile == LASSO_LOGIN_PROTOCOL_PROFILE_BRWS_POST)
 		xmlNewTextChild(xmlnode, NULL, (xmlChar*)"ProtocolProfile", (xmlChar*)"POST");
+	else if (login->protocolProfile == LASSO_LOGIN_PROTOCOL_PROFILE_REDIRECT)
+		xmlNewTextChild(xmlnode, NULL, (xmlChar*)"ProtocolProfile", (xmlChar*)"Redirect");
 
 	return xmlnode;
 }
@@ -1887,8 +2073,10 @@ init_from_xml(LassoNode *node, xmlNode *xmlnode)
 			s = (char*)xmlNodeGetContent(t);
 			if (strcmp(s, "Artifact") == 0)
 				login->protocolProfile = LASSO_LOGIN_PROTOCOL_PROFILE_BRWS_ART;
-			if (strcmp(s, "POST") == 0)
+			else if (strcmp(s, "POST") == 0)
 				login->protocolProfile = LASSO_LOGIN_PROTOCOL_PROFILE_BRWS_POST;
+			else if (strcmp(s, "Redirect") == 0)
+				login->protocolProfile = LASSO_LOGIN_PROTOCOL_PROFILE_REDIRECT;
 			xmlFree(s);
 		}
 		t = t->next;
@@ -1905,17 +2093,19 @@ static void
 dispose(GObject *object)
 {
 	LassoLogin *login = LASSO_LOGIN(object);
+
 	g_free(login->private_data->soap_request_msg);
 	login->private_data->soap_request_msg = NULL;
-	if (login->private_data->saml2_assertion)
-		lasso_node_destroy(LASSO_NODE(login->private_data->saml2_assertion));
+
+
+	lasso_node_destroy(LASSO_NODE(login->private_data->saml2_assertion));
 	login->private_data->saml2_assertion = NULL;
+
 #ifdef LASSO_WSF_ENABLED
-	if (login->private_data->resourceId)
-		lasso_node_destroy(LASSO_NODE(login->private_data->resourceId));
+	lasso_node_destroy(LASSO_NODE(login->private_data->resourceId));
 	login->private_data->resourceId = NULL;
-	if (login->private_data->encryptedResourceId)
-		lasso_node_destroy(LASSO_NODE(login->private_data->encryptedResourceId));
+
+	lasso_node_destroy(LASSO_NODE(login->private_data->encryptedResourceId));
 	login->private_data->encryptedResourceId = NULL;
 #endif
 	G_OBJECT_CLASS(parent_class)->dispose(object);
