@@ -1,4 +1,4 @@
-/* $Id: profile.c 3324 2007-06-19 11:16:17Z fpeters $
+/* $Id: profile.c 3729 2008-05-22 07:54:46Z dlaniel $
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -22,6 +22,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/**
+ * SECTION:profile
+ * @short_description: Base class for all identity profiles
+ *
+ **/
+
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
@@ -49,7 +55,7 @@
  * #LassoSaml2NameID in SAML 2.0).
  *
  * Return value: the name identifier or NULL if none was found.  The #LassoNode
- *     object is internally allocated and must not be *     freed by the caller.
+ *     object is internally allocated and must not be freed by the caller.
  **/
 LassoNode*
 lasso_profile_get_nameIdentifier(LassoProfile *profile)
@@ -346,6 +352,9 @@ lasso_profile_set_identity_from_dump(LassoProfile *profile, const gchar *dump)
 {
 	g_return_val_if_fail(dump != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
+	if (profile->identity) {
+		g_object_unref(profile->identity);
+	}
 	profile->identity = lasso_identity_new_from_dump(dump);
 	if (profile->identity == NULL)
 		return critical_error(LASSO_PROFILE_ERROR_BAD_IDENTITY_DUMP);
@@ -368,6 +377,9 @@ lasso_profile_set_session_from_dump(LassoProfile *profile, const gchar *dump)
 {
 	g_return_val_if_fail(dump != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
+	if (profile->session) {
+		g_object_unref(profile->session);
+	}
 	profile->session = lasso_session_new_from_dump(dump);
 	if (profile->session == NULL)
 		return critical_error(LASSO_PROFILE_ERROR_BAD_SESSION_DUMP);
@@ -499,12 +511,16 @@ dispose(GObject *object)
 
 	lasso_server_destroy(profile->server);
 	profile->server = NULL;
+
 	lasso_identity_destroy(profile->identity);
 	profile->identity = NULL;
+
 	lasso_session_destroy(profile->session);
 	profile->session = NULL;
+
 	g_free(profile->private_data->artifact);
 	profile->private_data->artifact = NULL;
+
 	g_free(profile->private_data->artifact_message);
 	profile->private_data->artifact_message = NULL;
 
