@@ -1,4 +1,4 @@
-/* $Id: data_service.c 3374 2007-08-12 22:19:32Z fpeters $
+/* $Id: data_service.c 3712 2008-05-20 13:14:06Z dlaniel $
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -22,6 +22,37 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/**
+ * SECTION:data_service
+ * @short_description: ID-WSF Data Service Profile
+ *
+ * Following up on #LassoDiscovery first example, it created a @service object,
+ * this is a #LassoDataService instance.  This example continues from that step
+ * and retrieves the name of the principal:
+ *  
+ * <informalexample>
+ * <programlisting>
+ * char *soap_answer;            // SOAP answer from data service
+ * xmlNode *principal_name;      // libxml2 xmlNode with the principal name
+ * 
+ * service = lasso_discovery_get_service(discovery);
+ * lasso_data_service_init_query(service, "/pp:PP/pp:InformalName", NULL);
+ * lasso_data_service_build_request_msg(service);
+ * 
+ * // service must perform SOAP call to LASSO_WSF_PROFILE(service)->msg_url
+ * // the SOAP message is LASSO_WSF_PROFILE(service)->msg_body.  The answer
+ * // is stored in char* soap_answer;
+ * 
+ * lasso_data_service_process_query_response_msg(service, soap_answer);
+ * principal_name = lasso_data_service_get_answer(service, "/pp:PP/pp:InformalName");
+ * 
+ * // app should probably then use xmlNodeGetContent libxml2 function to get
+ * // access to node content.
+ * </programlisting>
+ * </informalexample>
+ * 
+ */
+
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
@@ -41,6 +72,7 @@
 #include <xmlsec/templates.h>
 #include <xmlsec/crypto.h>
 
+extern GHashTable *dst_services_by_prefix; /* cf xml/xml.c */
 
 struct _LassoDataServicePrivate
 {
@@ -50,6 +82,8 @@ struct _LassoDataServicePrivate
 	LassoSoapFault *fault;
 };
 
+static void lasso_register_idwsf_xpath_namespaces(xmlXPathContext *xpathCtx);
+
 /*****************************************************************************/
 /* public methods                                                            */
 /*****************************************************************************/
@@ -64,81 +98,6 @@ lasso_data_service_add_credential(LassoDataService *service,
 	return 0;
 }
 
-LassoDstModification*
-lasso_data_service_add_modification(LassoDataService *service, const gchar *select)
-{
-	LassoWsfProfile *profile;
-	LassoDstModification *modification;
-
-	g_return_val_if_fail(LASSO_IS_PROFILE_SERVICE(service), NULL);
-	g_return_val_if_fail(select != NULL, NULL);
-
-	profile = LASSO_WSF_PROFILE(service);
-
-	modification = lasso_dst_modification_new(select);
-	LASSO_DST_MODIFY(profile->request)->Modification = g_list_append(
-		LASSO_DST_MODIFY(profile->request)->Modification, (gpointer)modification);
-
-	return modification;
-}
-
-
-/**
- * lasso_data_service_add_query_item:
- * @service: a #LassoDataService
- * @select: resource selection string (typically a XPath query)
- * @item_id: query item identifier
- *
- * Adds a dst:QueryItem to the current dst:Query request.
- *
- * Return value: a newly created #LassoDstQueryItem with the query item that
- *       has been created.  Note that it is internally allocated and shouldn't
- *       be freed by the caller.
- **/
-LassoDstQueryItem*
-lasso_data_service_add_query_item(LassoDataService *service,
-		const char *select, const char *item_id)
-{
-	LassoDstQuery *query;
-	LassoDstQueryItem *item;
-
-	g_return_val_if_fail(LASSO_IS_PROFILE_SERVICE(service), NULL);
-	g_return_val_if_fail(select != NULL, NULL);
-
-	if (! LASSO_IS_DST_QUERY(LASSO_WSF_PROFILE(service)->request)) {
-		return NULL;
-	}
-
-	query = LASSO_DST_QUERY(LASSO_WSF_PROFILE(service)->request);
-	
-	if (query->QueryItem && query->QueryItem->data && 
-			LASSO_DST_QUERY_ITEM(query->QueryItem->data)->itemID == NULL) {
-		/* XXX: all items must have itemID if there is more than one,
-		 * perhaps we could generate an item id for those lacking it */
-		return NULL;
-	}
-
-	item = lasso_dst_query_item_new(select, item_id);
-	query->QueryItem = g_list_append(query->QueryItem, item);
-
-	return item;
-}
-
-gint
-lasso_data_service_need_redirect_user(LassoDataService *service, const char *redirectUrl)
-{
-	LassoSoapDetail *detail;
-
-	/* Find a SOAP fault element */
-	service->private_data->fault = lasso_soap_fault_new();
-	service->private_data->fault->faultcode = g_strdup(LASSO_SOAP_FAULT_CODE_SERVER);
-	detail = lasso_soap_detail_new();
-	detail->any = g_list_append(detail->any, lasso_is_redirect_request_new(redirectUrl));
-	service->private_data->fault->Detail = detail;
-
-	return 0;
-}
-
 /**
  * lasso_data_service_init_query
  * @service: a #LassoDataService
@@ -187,7 +146,7 @@ lasso_data_service_init_query(LassoDataService *service, const char *select,
 	query->prefixServiceType = lasso_get_prefix_for_dst_service_href(
 		query->hrefServiceType);
 	if (query->prefixServiceType == NULL) {
-		return LASSO_PROFILE_ERROR_MISSING_SERVICE_TYPE;
+		return LASSO_DATA_SERVICE_ERROR_UNREGISTERED_DST;
 	}
 
 	if (offering->ResourceID) {
@@ -201,17 +160,18 @@ lasso_data_service_init_query(LassoDataService *service, const char *select,
 
 	lasso_wsf_profile_init_soap_request(LASSO_WSF_PROFILE(service), LASSO_NODE(query));
 
+	/* Set description */
 	if (security_mech_id == NULL) {
 		description = LASSO_DISCO_DESCRIPTION(offering->ServiceInstance->Description->data);
 	} else {
 		description = lasso_discovery_get_description_auto(offering, security_mech_id);
 	}
-
 	if (description == NULL) {
 		return LASSO_PROFILE_ERROR_MISSING_SERVICE_DESCRIPTION;
 	}
 	lasso_wsf_profile_set_description(LASSO_WSF_PROFILE(service), description);
 
+	/* Set msgUrl */
 	if (description->Endpoint != NULL) {
 		profile->msg_url = g_strdup(description->Endpoint);
 	} else {
@@ -237,49 +197,44 @@ lasso_data_service_init_query(LassoDataService *service, const char *select,
 }
 
 /**
- * lasso_data_service_get_redirect_request_url:
+ * lasso_data_service_add_query_item:
  * @service: a #LassoDataService
- * @message: the dst query message
+ * @select: resource selection string (typically a XPath query)
+ * @item_id: query item identifier
  *
- * Tells if Attribute Provider needs user interaction.
+ * Adds a dst:QueryItem to the current dst:Query request.
  *
- * Return value: TRUE if needed; or FALSE otherwise.
+ * Return value: a newly created #LassoDstQueryItem with the query item that
+ *       has been created.  Note that it is internally allocated and shouldn't
+ *       be freed by the caller.
  **/
-gchar*
-lasso_data_service_get_redirect_request_url(LassoDataService *service)
+LassoDstQueryItem*
+lasso_data_service_add_query_item(LassoDataService *service,
+		const char *select, const char *item_id)
 {
-	LassoSoapFault *fault = NULL;
-	LassoIsRedirectRequest *redirect_request = NULL;
-	GList *iter;
+	LassoDstQuery *query;
+	LassoDstQueryItem *item;
 
-	if (LASSO_WSF_PROFILE(service)->soap_envelope_response == NULL ||
-			LASSO_WSF_PROFILE(service)->soap_envelope_response->Body == NULL) {
+	g_return_val_if_fail(LASSO_IS_DATA_SERVICE(service), NULL);
+	g_return_val_if_fail(select != NULL, NULL);
+
+	if (! LASSO_IS_DST_QUERY(LASSO_WSF_PROFILE(service)->request)) {
 		return NULL;
 	}
 
-	iter = LASSO_WSF_PROFILE(service)->soap_envelope_response->Body->any;
-	while (iter) {
-		if (LASSO_IS_SOAP_FAULT(iter->data) == TRUE) {
-			fault = LASSO_SOAP_FAULT(iter->data);
-			break;
-		}
-		iter = iter->next;
-	}
-	if (fault == NULL || fault->Detail == NULL)
+	query = LASSO_DST_QUERY(LASSO_WSF_PROFILE(service)->request);
+	
+	if (query->QueryItem && query->QueryItem->data && 
+			LASSO_DST_QUERY_ITEM(query->QueryItem->data)->itemID == NULL) {
+		/* XXX: all items must have itemID if there is more than one,
+		 * perhaps we could generate an item id for those lacking it */
 		return NULL;
-
-	iter = fault->Detail->any;
-	while (iter) {
-		if (LASSO_IS_IS_REDIRECT_REQUEST(iter->data) == TRUE) {
-			redirect_request = LASSO_IS_REDIRECT_REQUEST(iter->data);
-			break;
-		}
-		iter = g_list_next(iter);
 	}
-	if (redirect_request == NULL)
-		return NULL;
 
-	return g_strdup(redirect_request->redirectURL);
+	item = lasso_dst_query_item_new(select, item_id);
+	query->QueryItem = g_list_append(query->QueryItem, item);
+
+	return item;
 }
 
 /**
@@ -302,7 +257,6 @@ lasso_data_service_process_query_msg(LassoDataService *service, const char *mess
 	gchar *service_type;
 	GList *node_list;
 	LassoSoapEnvelope *envelope;
-	LassoDstQuery *request;
 	xmlDoc *doc;
 	xmlNode *xmlnode;
 
@@ -323,18 +277,22 @@ lasso_data_service_process_query_msg(LassoDataService *service, const char *mess
 		xmlnode = NULL;
 	}
 
-	envelope = LASSO_SOAP_ENVELOPE(
-		lasso_node_new_from_xmlNode(xmlDocGetRootElement(doc)));
-	request = LASSO_DST_QUERY(envelope->Body->any->data);
-	service_type = g_strdup(request->hrefServiceType);
+	envelope = LASSO_SOAP_ENVELOPE(lasso_node_new_from_xmlNode(xmlDocGetRootElement(doc)));
+	if (envelope->Body == NULL || envelope->Body->any == NULL
+			|| envelope->Body->any->data == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_INVALID_MSG);
+	}
+	query = LASSO_DST_QUERY(envelope->Body->any->data);
+	service_type = g_strdup(query->hrefServiceType);
 	xmlFreeDoc(doc);
 
 	profile = LASSO_WSF_PROFILE(service);
 	rc = lasso_wsf_profile_process_soap_request_msg(profile, message, service_type,
 							security_mech_id);
-	if (rc)
+	if (rc) {
 		return rc;
-	
+	}
+
 	/* get provider id from soap:Header */
 	for (node_list = profile->soap_envelope_request->Header->Other;
 			node_list; node_list = g_list_next(node_list)) {
@@ -346,8 +304,7 @@ lasso_data_service_process_query_msg(LassoDataService *service, const char *mess
 				LASSO_SOAP_BINDING_PROVIDER(node)->providerID);
 		}
 	}
-	
-	query = LASSO_DST_QUERY(profile->request);
+
 	if (query->ResourceID) {
 		service->resource_id = g_object_ref(query->ResourceID);
 	} else if (query->EncryptedResourceID) {
@@ -359,58 +316,6 @@ lasso_data_service_process_query_msg(LassoDataService *service, const char *mess
 	return 0;
 }
 
-gint
-lasso_data_service_build_modify_response_msg(LassoDataService *service) {
-	LassoWsfProfile *profile;
-	LassoDstModify *request;
-	LassoDstModifyResponse *response;
-
-	GList *iter;
-	xmlDoc *doc;
-	xmlXPathContext *xpathCtx;
-	xmlXPathObject *xpathObj;
-
-	LassoSoapEnvelope *envelope;
-
-	profile = LASSO_WSF_PROFILE(service);
-	request = LASSO_DST_MODIFY(profile->request);
-
-	response = lasso_dst_modify_response_new(
-		lasso_utility_status_new(LASSO_DST_STATUS_CODE_OK));
-	profile->response = LASSO_NODE(response);
-	response->prefixServiceType = g_strdup(request->prefixServiceType);
-	response->hrefServiceType = g_strdup(request->hrefServiceType);
-	envelope = profile->soap_envelope_response;
-	envelope->Body->any = g_list_append(envelope->Body->any, response);
-
-	doc = xmlNewDoc((xmlChar*)"1.0");
-	xmlDocSetRootElement(doc, service->resource_data);
-	xpathCtx = xmlXPathNewContext(doc);
-	xmlXPathRegisterNs(xpathCtx, (xmlChar*)response->prefixServiceType,
-			(xmlChar*)response->hrefServiceType);
-
-	iter = request->Modification;
-	while (iter) {
-		LassoDstModification *modification = iter->data;
-		xmlNode *newNode = modification->NewData->any->data;
-		xpathObj = xmlXPathEvalExpression((xmlChar*)modification->Select,
-			xpathCtx);
-		if (xpathObj && xpathObj->nodesetval && xpathObj->nodesetval->nodeNr) {
-			xmlNode *node = xpathObj->nodesetval->nodeTab[0];
-			xmlReplaceNode(node, newNode);
-		}
-		xmlXPathFreeObject(xpathObj);
-		xpathObj = NULL;
-
-		iter = g_list_next(iter);
-	}
-
-	xmlXPathFreeContext(xpathCtx);
-	xmlFreeDoc(doc);
-
-	return lasso_wsf_profile_build_soap_response_msg(profile);
-}
-
 /**
  * lasso_data_service_build_response_msg:
  * @service: a #LassoDataService
@@ -452,25 +357,35 @@ lasso_data_service_build_response_msg(LassoDataService *service)
 	doc = xmlNewDoc((xmlChar*)"1.0");
 	xmlDocSetRootElement(doc, service->resource_data);
 	xpathCtx = xmlXPathNewContext(doc);
-	xmlXPathRegisterNs(xpathCtx, (xmlChar*)response->prefixServiceType,
-			(xmlChar*)response->hrefServiceType);
+	lasso_register_idwsf_xpath_namespaces(xpathCtx);
 
 	/* XXX: needs another level, since there may be more than one <dst:Query> */
 	iter = request->QueryItem;
 	while (iter) {
 		LassoDstQueryItem *item = iter->data;
+		LassoDstData *data;
+
 		xpathObj = xmlXPathEvalExpression((xmlChar*)item->Select, xpathCtx);
 		if (xpathObj && xpathObj->nodesetval && xpathObj->nodesetval->nodeNr) {
-			LassoDstData *data;
 			xmlNode *node = xpathObj->nodesetval->nodeTab[0];
 			/* XXX: assuming there is only one matching node */
 			data = lasso_dst_data_new();
 			data->any = g_list_append(data->any, xmlCopyNode(node, 1));
-			if (item->itemID) {
-				data->itemIDRef = g_strdup(item->itemID);
+		} else if (xpathObj && xpathObj->type == XPATH_STRING) {
+			data = lasso_dst_data_new();
+			data->any = g_list_append(data->any,
+					xmlNewText(xpathObj->stringval));
+		} else {
+			/* no response was found, break here */
+			if (xpathObj) {
+				xmlXPathFreeObject(xpathObj);
 			}
-			response->Data = g_list_append(response->Data, data);
+			break;
 		}
+		if (item->itemID) {
+			data->itemIDRef = g_strdup(item->itemID);
+		}
+		response->Data = g_list_append(response->Data, data);
 		xmlXPathFreeObject(xpathObj);
 		xpathObj = NULL;
 		iter = g_list_next(iter);
@@ -642,6 +557,66 @@ lasso_data_service_process_query_response_msg(LassoDataService *service,
 	return 0;
 }
 
+gint
+lasso_data_service_need_redirect_user(LassoDataService *service, const char *redirectUrl)
+{
+	LassoSoapDetail *detail;
+
+	/* Find a SOAP fault element */
+	service->private_data->fault = lasso_soap_fault_new();
+	service->private_data->fault->faultcode = g_strdup(LASSO_SOAP_FAULT_CODE_SERVER);
+	detail = lasso_soap_detail_new();
+	detail->any = g_list_append(detail->any, lasso_is_redirect_request_new(redirectUrl));
+	service->private_data->fault->Detail = detail;
+
+	return 0;
+}
+
+/**
+ * lasso_data_service_get_redirect_request_url:
+ * @service: a #LassoDataService
+ * @message: the dst query message
+ *
+ * Tells if Attribute Provider needs user interaction.
+ *
+ * Return value: TRUE if needed; or FALSE otherwise.
+ **/
+gchar*
+lasso_data_service_get_redirect_request_url(LassoDataService *service)
+{
+	LassoSoapFault *fault = NULL;
+	LassoIsRedirectRequest *redirect_request = NULL;
+	GList *iter;
+
+	if (LASSO_WSF_PROFILE(service)->soap_envelope_response == NULL ||
+			LASSO_WSF_PROFILE(service)->soap_envelope_response->Body == NULL) {
+		return NULL;
+	}
+
+	iter = LASSO_WSF_PROFILE(service)->soap_envelope_response->Body->any;
+	while (iter) {
+		if (LASSO_IS_SOAP_FAULT(iter->data) == TRUE) {
+			fault = LASSO_SOAP_FAULT(iter->data);
+			break;
+		}
+		iter = iter->next;
+	}
+	if (fault == NULL || fault->Detail == NULL)
+		return NULL;
+
+	iter = fault->Detail->any;
+	while (iter) {
+		if (LASSO_IS_IS_REDIRECT_REQUEST(iter->data) == TRUE) {
+			redirect_request = LASSO_IS_REDIRECT_REQUEST(iter->data);
+			break;
+		}
+		iter = g_list_next(iter);
+	}
+	if (redirect_request == NULL)
+		return NULL;
+
+	return g_strdup(redirect_request->redirectURL);
+}
 
 gint
 lasso_data_service_init_modify(LassoDataService *service, const gchar *select,
@@ -652,10 +627,12 @@ lasso_data_service_init_modify(LassoDataService *service, const gchar *select,
 	LassoDiscoResourceOffering *offering;
 	LassoDiscoDescription *description;
 	LassoWsfProfile *profile;
-
-	LassoSoapEnvelope *envelope;
 	LassoDstModify *modify;
 
+	g_return_val_if_fail(LASSO_IS_DATA_SERVICE(service),
+			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(service != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
+	g_return_val_if_fail(xmlData != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_WSF_PROFILE(service);
 
@@ -668,8 +645,16 @@ lasso_data_service_init_modify(LassoDataService *service, const gchar *select,
 	modify = lasso_dst_modify_new(modification);
 	profile->request = LASSO_NODE(modify);
 
+	if (service == NULL || service->private_data == NULL
+			|| service->private_data->offering == NULL) {
+		return LASSO_PROFILE_ERROR_MISSING_RESOURCE_OFFERING;
+	}
 	offering = service->private_data->offering;
 	
+	if (offering->ServiceInstance == NULL
+			|| offering->ServiceInstance->ServiceType == NULL) {
+		return LASSO_PROFILE_ERROR_MISSING_SERVICE_TYPE;
+	}
 	modify->hrefServiceType = g_strdup(offering->ServiceInstance->ServiceType);
 	modify->prefixServiceType = lasso_get_prefix_for_dst_service_href(
 			modify->hrefServiceType);
@@ -681,25 +666,130 @@ lasso_data_service_init_modify(LassoDataService *service, const gchar *select,
 	if (offering->ResourceID) {
 		modify->ResourceID = offering->ResourceID;
 	} else if (offering->EncryptedResourceID) {
-	  modify->EncryptedResourceID = offering->EncryptedResourceID;
+	        modify->EncryptedResourceID = offering->EncryptedResourceID;
 	} else {
 		/* XXX: no resource id, implied:resource, etc. */
 		return LASSO_ERROR_UNIMPLEMENTED;
 	}
 
-	envelope = lasso_wsf_profile_build_soap_envelope(NULL, NULL);
-	LASSO_WSF_PROFILE(service)->soap_envelope_request = envelope;
-	envelope->Body->any = g_list_append(envelope->Body->any, modify);
+	lasso_wsf_profile_init_soap_request(LASSO_WSF_PROFILE(service), LASSO_NODE(modify));
 
-	/* set msg_url */
-	/* TODO : implement WSDLRef */
-	if (description->Endpoint) {
+	/* Set description */
+	if (offering->ServiceInstance != NULL && offering->ServiceInstance->Description != NULL) {
+		description = LASSO_DISCO_DESCRIPTION(offering->ServiceInstance->Description->data);
+	}
+	if (description == NULL) {
+		return LASSO_PROFILE_ERROR_MISSING_SERVICE_DESCRIPTION;
+	}
+	lasso_wsf_profile_set_description(LASSO_WSF_PROFILE(service), description);
+
+	/* Set msgUrl */
+	if (description->Endpoint != NULL) {
 		profile->msg_url = g_strdup(description->Endpoint);
+	} else {
+		/* XXX: else, description->WsdlURLI, get endpoint automatically */
+		return LASSO_ERROR_UNIMPLEMENTED;
 	}
 
 	return 0;
 }
 
+LassoDstModification*
+lasso_data_service_add_modification(LassoDataService *service, const gchar *select)
+{
+	LassoWsfProfile *profile;
+	LassoDstModification *modification;
+
+	g_return_val_if_fail(LASSO_IS_DATA_SERVICE(service), NULL);
+	g_return_val_if_fail(select != NULL, NULL);
+
+	profile = LASSO_WSF_PROFILE(service);
+
+	modification = lasso_dst_modification_new(select);
+	LASSO_DST_MODIFY(profile->request)->Modification = g_list_append(
+		LASSO_DST_MODIFY(profile->request)->Modification, (gpointer)modification);
+
+	return modification;
+}
+
+gint
+lasso_data_service_build_modify_response_msg(LassoDataService *service)
+{
+	LassoWsfProfile *profile;
+	LassoDstModify *request;
+	LassoDstModifyResponse *response;
+	LassoSoapEnvelope *envelope;
+	GList *iter;
+	xmlNode *cur_data;
+	xmlDoc *doc;
+	xmlXPathContext *xpathCtx;
+	xmlXPathObject *xpathObj;
+	int res = 0;
+
+	profile = LASSO_WSF_PROFILE(service);
+	request = LASSO_DST_MODIFY(profile->request);
+
+	if (service->private_data->fault != NULL) {
+		envelope = profile->soap_envelope_response;
+		envelope->Body->any = g_list_append(
+			envelope->Body->any, service->private_data->fault);
+		return lasso_wsf_profile_build_soap_response_msg(profile);
+	}
+
+	if (service->resource_data == NULL) {
+		return LASSO_DST_ERROR_MISSING_SERVICE_DATA;
+	} else {
+		cur_data = xmlCopyNode(service->resource_data, 1);
+	}
+
+	response = lasso_dst_modify_response_new(
+		lasso_utility_status_new(LASSO_DST_STATUS_CODE_OK));
+	profile->response = LASSO_NODE(response);
+	response->prefixServiceType = g_strdup(request->prefixServiceType);
+	response->hrefServiceType = g_strdup(request->hrefServiceType);
+	envelope = profile->soap_envelope_response;
+	envelope->Body->any = g_list_append(envelope->Body->any, response);
+
+	doc = xmlNewDoc((xmlChar*)"1.0");
+	xmlDocSetRootElement(doc, cur_data);
+	xpathCtx = xmlXPathNewContext(doc);
+	lasso_register_idwsf_xpath_namespaces(xpathCtx);
+
+	for (iter = request->Modification; iter != NULL; iter = g_list_next(iter)) {
+		LassoDstModification *modification = iter->data;
+		xmlNode *newNode = modification->NewData->any->data;
+		xpathObj = xmlXPathEvalExpression((xmlChar*)modification->Select,
+			xpathCtx);
+		if (xpathObj && xpathObj->nodesetval && xpathObj->nodesetval->nodeNr) {
+			xmlNode *node = xpathObj->nodesetval->nodeTab[0];
+			if (node != NULL) {
+				/* If we must replace the root element, change it in the xmlDoc */
+				if (node == cur_data) {
+					xmlDocSetRootElement(doc, newNode);
+					xmlFreeNode(cur_data);
+					cur_data = NULL;
+				} else {
+					xmlReplaceNode(node, newNode);
+				}
+			}
+		} else {
+			res = LASSO_DST_ERROR_MODIFY_FAILED;
+		}
+		xmlXPathFreeObject(xpathObj);
+		xpathObj = NULL;
+	}
+
+	if (res == 0 && doc->children != NULL) {
+		/* Save new service resource data */
+		xmlFreeNode(service->resource_data);
+		service->resource_data = xmlCopyNode(doc->children, 1);
+	}
+
+	xmlXPathFreeContext(xpathCtx);
+	xmlFreeDoc(doc);
+
+	return lasso_wsf_profile_build_soap_response_msg(profile);
+}
 
 gint
 lasso_data_service_process_modify_msg(LassoDataService *service,
@@ -707,9 +797,27 @@ lasso_data_service_process_modify_msg(LassoDataService *service,
 {
 	LassoDstModify *modify;
 	LassoWsfProfile *profile;
+	LassoSoapEnvelope *envelope;
+	xmlDoc *doc;
 	int rc;
 	gchar *service_type;
 
+
+	doc = lasso_xml_parse_memory(modify_soap_msg, strlen(modify_soap_msg));
+	if (doc == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_INVALID_MSG);
+	}
+
+	envelope = LASSO_SOAP_ENVELOPE(lasso_node_new_from_xmlNode(xmlDocGetRootElement(doc)));
+	if (envelope->Body == NULL || envelope->Body->any == NULL
+			|| envelope->Body->any->data == NULL) {
+		return critical_error(LASSO_PROFILE_ERROR_INVALID_MSG);
+	}
+
+	modify = LASSO_DST_MODIFY(envelope->Body->any->data);
+	service_type = g_strdup(modify->hrefServiceType);
+	xmlFreeDoc(doc);
+
 	profile = LASSO_WSF_PROFILE(service);
 	rc = lasso_wsf_profile_process_soap_request_msg(profile, modify_soap_msg, service_type,
 							security_mech_id);
@@ -717,7 +825,6 @@ lasso_data_service_process_modify_msg(LassoDataService *service,
 		return rc;
 	}
 
-	modify = LASSO_DST_MODIFY(profile->request);
 	if (modify->ResourceID) {
 		service->resource_id = g_object_ref(modify->ResourceID);
 	} else if (modify->EncryptedResourceID) {
@@ -726,27 +833,52 @@ lasso_data_service_process_modify_msg(LassoDataService *service,
 		return LASSO_ERROR_UNIMPLEMENTED; /* implied ? */
 	}
 
-	return 0;}
+	return 0;
+}
 
+/**
+ * lasso_data_service_process_modify_response_msg
+ * @service: a #LassoDataService
+ * @soap_msg: the SOAP message
+ *
+ * Return value: 0 on success; or a negative value otherwise.
+ **/
 gint
 lasso_data_service_process_modify_response_msg(LassoDataService *service, const gchar *soap_msg)
 {
 	LassoDstModifyResponse *response;
 	LassoSoapEnvelope *envelope;
 
-	g_return_val_if_fail(LASSO_IS_PROFILE_SERVICE(service),
+	g_return_val_if_fail(LASSO_IS_DATA_SERVICE(service),
 			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
 	g_return_val_if_fail(soap_msg != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	envelope = LASSO_SOAP_ENVELOPE(lasso_node_new_from_dump(soap_msg));
-	LASSO_WSF_PROFILE(service)->soap_envelope_response = envelope;
+	if (envelope == NULL || ! envelope->Body || ! envelope->Body->any || ! LASSO_IS_NODE(envelope->Body->any->data)) {
+		return critical_error(LASSO_PROFILE_ERROR_INVALID_MSG);
+	}
 
+	LASSO_WSF_PROFILE(service)->soap_envelope_response = envelope;
 	response = envelope->Body->any->data;
 	LASSO_WSF_PROFILE(service)->response = LASSO_NODE(response);
 
 	return 0;
 }
 
+/**
+ * lasso_data_service_get_resource_offering:
+ * @service: a #LassoDataService
+ *
+ * Gets the #LassoDiscoResourceOffering of the @service.
+ *
+ * Return value: the #LassoDiscoResourceOffering associated to service.
+ **/
+LassoDiscoResourceOffering*
+lasso_data_service_get_resource_offering(LassoDataService *service)
+{
+	return g_object_ref(service->private_data->offering);
+}
+
 
 /*****************************************************************************/
 /* private methods                                                           */
@@ -755,12 +887,40 @@ lasso_data_service_process_modify_response_msg(LassoDataService *service, const
 static LassoNodeClass *parent_class = NULL;
 
 void
-lasso_data_service_set_offering(LassoDataService *service,
-		LassoDiscoResourceOffering *offering)
+lasso_data_service_set_offering(LassoDataService *service, LassoDiscoResourceOffering *offering)
 {
 	service->private_data->offering = g_object_ref(offering);
+	if (offering->ResourceID != NULL) {
+		service->resource_id = g_object_ref(offering->ResourceID);
+	}
+	if (offering->EncryptedResourceID != NULL) {
+		service->encrypted_resource_id = g_object_ref(offering->EncryptedResourceID);
+	}
+	service->provider_id = g_strdup(offering->ServiceInstance->ProviderID);
+	service->abstract_description = g_strdup(offering->Abstract);
 }
 
+static void
+register_xpath_namespace(gchar *prefix, gchar *href, xmlXPathContext *xpathCtx)
+{
+	xmlXPathRegisterNs(xpathCtx, (xmlChar*)prefix, (xmlChar*)href);
+}
+
+static void
+lasso_register_idwsf_xpath_namespaces(xmlXPathContext *xpathCtx)
+{
+	xmlXPathRegisterNs(xpathCtx, (xmlChar*)LASSO_PP_PREFIX,
+			(xmlChar*)LASSO_PP_HREF);
+	xmlXPathRegisterNs(xpathCtx, (xmlChar*)LASSO_EP_PREFIX,
+			(xmlChar*)LASSO_EP_HREF);
+	if (dst_services_by_prefix == NULL)
+		return;
+	g_hash_table_foreach(dst_services_by_prefix,
+			(GHFunc)register_xpath_namespace, xpathCtx);
+}
+
+
+
 
 /*****************************************************************************/
 /* overrided parent class methods */
@@ -854,9 +1014,9 @@ lasso_data_service_new(LassoServer *server)
 {
 	LassoDataService *service;
 
-	g_return_val_if_fail(LASSO_IS_SERVER(server) == TRUE, NULL);
+	g_return_val_if_fail(LASSO_IS_SERVER(server), NULL);
 
-	service = g_object_new(LASSO_TYPE_PROFILE_SERVICE, NULL);
+	service = g_object_new(LASSO_TYPE_DATA_SERVICE, NULL);
 	LASSO_WSF_PROFILE(service)->server = g_object_ref(server);
 
 	return service;
@@ -865,13 +1025,15 @@ lasso_data_service_new(LassoServer *server)
 LassoDataService*
 lasso_data_service_new_full(LassoServer *server, LassoDiscoResourceOffering *offering)
 {
-	LassoDataService *service;
+	LassoDataService *service = lasso_data_service_new(server);
 
-	service = lasso_data_service_new(server);
-	if (service == NULL)
+	g_return_val_if_fail(LASSO_IS_DISCO_RESOURCE_OFFERING(offering), NULL);
+	
+	if (service == NULL) {
 		return NULL;
+	}
 
-	service->private_data->offering = g_object_ref(offering);
+	lasso_data_service_set_offering(LASSO_DATA_SERVICE(service), offering);
 
 	return service;
 }
