@@ -1,4 +1,4 @@
-/* $Id: name_id_management.c 3237 2007-05-30 17:17:45Z dlaniel $ 
+/* $Id: name_id_management.c 3725 2008-05-21 17:28:44Z dlaniel $ 
  *
  * Lasso - A free implementation of the Liberty Alliance specifications.
  *
@@ -31,6 +31,12 @@
 #include <lasso/id-ff/serverprivate.h>
 #include <lasso/xml/xml_enc.h>
 
+/**
+ * SECTION:name_id_management
+ * @short_description: Name Id Management Profile (SAMLv2)
+ *
+ **/
+
 /*****************************************************************************/
 /* public methods                                                            */
 /*****************************************************************************/
@@ -58,10 +64,11 @@ lasso_name_id_management_init_request(LassoNameIdManagement *name_id_management,
 	LassoFederation *federation;
 	LassoSaml2NameID *name_id, *name_id_n;
 	LassoSamlp2RequestAbstract *request;
+	LassoSession *session = NULL;
+	LassoNode *oldNameIdentifier;
 
 	g_return_val_if_fail(LASSO_IS_NAME_ID_MANAGEMENT(name_id_management),
 			LASSO_PARAM_ERROR_INVALID_VALUE);
-	g_return_val_if_fail(remote_provider_id != NULL, LASSO_PARAM_ERROR_INVALID_VALUE);
 
 	profile = LASSO_PROFILE(name_id_management);
 
@@ -71,7 +78,17 @@ lasso_name_id_management_init_request(LassoNameIdManagement *name_id_management,
 	}
 
 	/* set the remote provider id */
-	profile->remote_providerID = g_strdup(remote_provider_id);
+	g_free (profile->remote_providerID);
+	if (remote_provider_id == NULL) {
+		/* verify if session exists */
+		session = lasso_profile_get_session(profile);
+		if (session == NULL) {
+			return critical_error(LASSO_PROFILE_ERROR_SESSION_NOT_FOUND);
+		}
+		profile->remote_providerID = lasso_session_get_provider_index(session, 0);
+	} else {
+		profile->remote_providerID = g_strdup(remote_provider_id);
+	}
 
 	remote_provider = g_hash_table_lookup(profile->server->providers,
 			profile->remote_providerID);
@@ -88,14 +105,15 @@ lasso_name_id_management_init_request(LassoNameIdManagement *name_id_management,
 
 	/* get the current NameID */
 	name_id_n = LASSO_SAML2_NAME_ID(lasso_profile_get_nameIdentifier(profile));
-
 	name_id = LASSO_SAML2_NAME_ID(name_id_n);
-
+	oldNameIdentifier = profile->nameIdentifier;
 	if (federation->local_nameIdentifier) {
 		profile->nameIdentifier = g_object_ref(federation->local_nameIdentifier);
 	} else {
 		profile->nameIdentifier = g_object_ref(name_id_n);
 	}
+	if (oldNameIdentifier != NULL) 
+		g_object_unref(oldNameIdentifier);
 
 	/* XXX: check HTTP method is supported */
 
@@ -208,6 +226,8 @@ lasso_name_id_management_process_request_msg(LassoNameIdManagement *name_id_mana
 	xmlSecKey *encryption_private_key = NULL;
 
 	g_return_val_if_fail(LASSO_IS_NAME_ID_MANAGEMENT(name_id_management),
+			LASSO_PARAM_ERROR_BAD_TYPE_OR_NULL_OBJ);
+	g_return_val_if_fail(request_msg != NULL, 
 			LASSO_PARAM_ERROR_INVALID_VALUE);
 	
 	profile = LASSO_PROFILE(name_id_management);
@@ -254,8 +274,9 @@ lasso_name_id_management_process_request_msg(LassoNameIdManagement *name_id_mana
 				encrypted_id, encryption_private_key));
 			LASSO_SAMLP2_MANAGE_NAME_ID_REQUEST(profile->request)->NameID = \
 				LASSO_SAML2_NAME_ID(profile->nameIdentifier);
+			g_object_unref(
+				LASSO_SAMLP2_MANAGE_NAME_ID_REQUEST(profile->request)->EncryptedID);
 			LASSO_SAMLP2_MANAGE_NAME_ID_REQUEST(profile->request)->EncryptedID = NULL;
-
 		}
 	} else {
 		profile->nameIdentifier = g_object_ref(name_id);
